//-----------------------------------------------------------------------------
// File: D3DFont.cpp
//-----------------------------------------------------------------------------

#include <d3dx9math.h>
#include "D3DFont.h"


//-----------------------------------------------------------------------------
// Custom vertex types for rendering text
//-----------------------------------------------------------------------------
#define MAX_NUM_VERTICES 50*6

struct FONT2DVERTEX { D3DXVECTOR4 p;   DWORD color;    FLOAT tu, tv; };
struct FONT3DVERTEX { D3DXVECTOR3 p;   D3DXVECTOR3 n;   FLOAT tu, tv; };

#define D3DFVF_FONT2DVERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_TEX1)
#define D3DFVF_FONT3DVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1)


inline FONT2DVERTEX InitFont2DVertex(const D3DXVECTOR4& p, D3DCOLOR color,
	FLOAT tu, FLOAT tv)
{
	FONT2DVERTEX v;   v.p = p;   v.color = color;   v.tu = tu;   v.tv = tv;

	return v;
}

inline FONT3DVERTEX InitFont3DVertex(const D3DXVECTOR3& p, const D3DXVECTOR3& n,
	FLOAT tu, FLOAT tv)
{
	FONT3DVERTEX v;   v.p = p;   v.n = n;   v.tu = tu;   v.tv = tv;

	return v;
}


//-----------------------------------------------------------------------------
// Name: CD3DFont()
// Desc: Font class constructor
//-----------------------------------------------------------------------------
CD3DFont::CD3DFont(const TCHAR* strFontName, DWORD dwHeight, DWORD dwFlags)
{
	_tcsncpy_s(m_strFontName, strFontName, sizeof(m_strFontName) / sizeof(TCHAR));

	// ڵ̸ wcscpy, ƴϸ strcpy óϴ Լ.
	// MultiByteToWideChar().
	m_strFontName[sizeof(m_strFontName) / sizeof(TCHAR)-1] = _T('\0');

	// ڿ  . _T ڵ ڵ , ƴϸ ANSIڿ ϴ ũ.

	m_dwFontHeight = dwHeight;
	m_dwFontFlags = dwFlags;

	m_pd3dDevice = NULL;
	m_pTexture = NULL;
	m_pVB = NULL;

	m_pStateBlockSaved = NULL;
	m_pStateBlockDrawText = NULL;
}

//-----------------------------------------------------------------------------
// Name: ~CD3DFont()
// Desc: Font class destructor
//-----------------------------------------------------------------------------
CD3DFont::~CD3DFont()
{
	InvalidateDeviceObjects();
	DeleteDeviceObjects();
}

//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Destroys all device-dependent objects
//-----------------------------------------------------------------------------
HRESULT CD3DFont::InvalidateDeviceObjects()
{
	// COM .
	SAFE_RELEASE(m_pVB);
	SAFE_RELEASE(m_pStateBlockSaved);
	SAFE_RELEASE(m_pStateBlockDrawText);

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Destroys all device-dependent objects
//-----------------------------------------------------------------------------
HRESULT CD3DFont::DeleteDeviceObjects()
{
	// COM  ʱȭ.
	SAFE_RELEASE(m_pTexture);
	m_pd3dDevice = NULL;

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initializes device-dependent objects, including the vertex buffer used
//       for rendering text and the texture map which stores the font image.
//-----------------------------------------------------------------------------
HRESULT CD3DFont::InitDeviceObjects(LPDIRECT3DDEVICE9 pd3dDevice)
{
	HRESULT hr;

	// ޹ ̽ 
	m_pd3dDevice = pd3dDevice;


	// ؽƮ ũ 
	m_fTextScale = 1.0f;

	// ؽƮ ̿ ° ʺ .
	if (m_dwFontHeight > 40)
		m_dwTexWidth = m_dwTexHeight = 1024;
	else if (m_dwFontHeight > 20)
		m_dwTexWidth = m_dwTexHeight = 512;
	else
		m_dwTexWidth = m_dwTexHeight = 256;

	// ̽  .
	D3DCAPS9 d3dCaps;
	m_pd3dDevice->GetDeviceCaps(&d3dCaps);

	if (m_dwTexWidth > d3dCaps.MaxTextureWidth)
	{
		m_fTextScale = (FLOAT)d3dCaps.MaxTextureWidth / (FLOAT)m_dwTexWidth;
		m_dwTexWidth = m_dwTexHeight = d3dCaps.MaxTextureWidth;
	}
	
	// ؽó (pDevice->(UINT Width,UINT Height,UINT Levels,DWORD Usage,D3DFORMAT Format,
	//            D3DPOOL Pool,IDirect3DTexture9** ppTexture,HANDLE* pSharedHandle)
	hr = m_pd3dDevice->CreateTexture(m_dwTexWidth, m_dwTexHeight, 1,
		0, D3DFMT_A4R4G4B4, D3DPOOL_MANAGED, &m_pTexture, 0);

	if (FAILED(hr))
		return hr;

	// Prepare to create a bitmap
	DWORD*      pBitmapBits;
	BITMAPINFO bmi;											// Ʈ   ü.
	ZeroMemory(&bmi.bmiHeader, sizeof(BITMAPINFOHEADER));   // 0 δ.
	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);        // 
	bmi.bmiHeader.biWidth = (int)m_dwTexWidth;				// ʺ
	bmi.bmiHeader.biHeight = -(int)m_dwTexHeight;           // ũ
	bmi.bmiHeader.biPlanes = 1;								// ÷ΰ  
	BI_RGB;													// 
	bmi.bmiHeader.biBitCount = 32;                          // Ʈ ̼

	// ޸ DC  ִ Լ. hDC NULL .
	// NULL ̸  ȭ ȣȯǴ DC .
	HDC    hDC = CreateCompatibleDC(NULL);

	// CreateCompatibleDC 16M  ֱ⿡ ̰ ٴµ ٰ .
	// ׳ Ʈ ̶ .
	// pBitmapBits: Ʈ Ʈ  ġ  ͸ ޴ .
	HBITMAP hbmBitmap = CreateDIBSection(hDC, &bmi, DIB_RGB_COLORS,
			(void**)&pBitmapBits, NULL, 0);

	// θ (GDIԼ  1  1pixel)
	SetMapMode(hDC, MM_TEXT);

	// Create a font.  By specifying ANTIALIASED_QUALITY, we might get an
	// antialiased font, but this is not guaranteed.

	// MulDiv(): ڷ ޹ 32Ʈ    64Ʈ ϰ,
	// ٽ 32Ʈ  Ȱ(m_dwFontHeight̿ شϴ ȼ  )
	// (m_dwFontHeight * (ġ  * Ʈ ũ)) / 72
	// GetDeviceCaps(hDC, LOGPIXELSY):  ػ 
	// 72 1ġ ȼ 
	INT nHeight = -MulDiv(m_dwFontHeight,
		(INT)(GetDeviceCaps(hDC, LOGPIXELSY) * m_fTextScale), 72);

	//  .
	DWORD dwBold = (m_dwFontFlags&D3DFONT_BOLD) ? FW_BOLD : FW_NORMAL;

	// Ÿü .
	DWORD dwItalic = (m_dwFontFlags&D3DFONT_ITALIC) ? TRUE : FALSE;

	// (cHeight, cWidth, cEscapement, cOrientation, cWeight, bItalic, bUnderline, bStrikeOut, iCharSet,
	// iOutPrecision, iClipPrecision, iQuality, iPitchAndFamily, pszFaceName);
	// , , , , , Ÿü, , Ҽ, ڼ, Ȯ, Ŭ Ȯ,  , ڰ Ʈ, 
	HFONT hFont = CreateFont(nHeight, 0, 0, 0, dwBold, dwItalic,
		FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
		CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY,
		VARIABLE_PITCH, m_strFontName);

	if (NULL == hFont)
		return E_FAIL;

	// hbmBitmap, hFont DC Ѵ.
	SelectObject(hDC, hbmBitmap);
	SelectObject(hDC, hFont);

	// Set text properties
	//  .
	SetTextColor(hDC, RGB(255, 255, 255));

	//   .
	SetBkColor(hDC, 0x00000000);

	// .
	SetTextAlign(hDC, TA_TOP);

	// Loop through all printable character and output them to the bitmap..
	// Meanwhile, keep track of the corresponding tex coords for each character.
	DWORD x = 0;
	DWORD y = 0;
	TCHAR str[2] = _T("x");
	SIZE size;

	for (TCHAR c = 32; c < 127; c++)
	{
		str[0] = c;

		// ڿ ũ ϴ Լ(DC, ڿ, ڿ , out).
		GetTextExtentPoint32(hDC, str, 1, &size);

		// x ʺ񺸴 ũ x 0, y +1
		if ((DWORD)(x + size.cx + 1) > m_dwTexWidth)
		{
			x = 0;
			y += size.cy + 1;
		}

		// ǥ  Բ ϴ  Լ.
		// ETO_OPAQUE(    )
		// DC, xǥ, yǥ,  ɼ, ڽ , ڿ,  ,  迭.
		ExtTextOut(hDC, x + 0, y + 0, ETO_OPAQUE, NULL, str, 1, NULL);

		//  ڵ  .
		m_fTexCoords[c - 32][0] = ((FLOAT)(x + 0)) / m_dwTexWidth;
		m_fTexCoords[c - 32][1] = ((FLOAT)(y + 0)) / m_dwTexHeight;
		m_fTexCoords[c - 32][2] = ((FLOAT)(x + 0 + size.cx)) / m_dwTexWidth;
		m_fTexCoords[c - 32][3] = ((FLOAT)(y + 0 + size.cy)) / m_dwTexHeight;

		x += size.cx + 1;
	}

	// Lock the surface and write the alpha values for the set pixels
	//  2D(簢) ̹ ͸ ϴ ü.
	// D3DLOCKED_RECT: ǥ ȼ ͷ ͸  ĥ,  ۾ .
	D3DLOCKED_RECT d3dlr;

	// ؽó 簢 ٴ.
	m_pTexture->LockRect(0, &d3dlr, 0, 0);

	//   Ʈ ͸ (*ڸ).
	BYTE* pDstRow = (BYTE*)d3dlr.pBits;

	WORD* pDst16;
	BYTE bAlpha;	// 4-bit measure of pixel intensity

	for (y = 0; y < m_dwTexHeight; y++)
	{
		pDst16 = (WORD*)pDstRow;

		for (x = 0; x < m_dwTexWidth; x++)
		{
			bAlpha = (BYTE)((pBitmapBits[m_dwTexWidth*y + x] & 0xff) >> 4);

			if (bAlpha > 0)
			{
				*pDst16++ = (bAlpha << 12) | 0x0fff;
			}
			else
			{
				*pDst16++ = 0x0000;
			}
		}

		// ش Ʈ ǥ ġ .
		pDstRow += d3dlr.Pitch;
	}

	// Done updating texture, so clean up used objects
	m_pTexture->UnlockRect(0);
	DeleteObject(hbmBitmap);
	DeleteDC(hDC);
	DeleteObject(hFont);

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc:
//-----------------------------------------------------------------------------
HRESULT CD3DFont::RestoreDeviceObjects()
{
	HRESULT hr;

	// Create vertex buffer for the letters
	// ū  .
	int vertexSize = max(sizeof(FONT2DVERTEX), sizeof(FONT3DVERTEX));
	
	// ؽ  .
	if ( FAILED ( hr = m_pd3dDevice->CreateVertexBuffer(MAX_NUM_VERTICES * vertexSize,
		D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC, 0,
		D3DPOOL_DEFAULT, &m_pVB, 0) ) )
	{
		return hr;
	}

	// Create the state blocks for rendering text
	for (UINT which = 0; which<2; which++)
	{
		// ̽    .
		m_pd3dDevice->BeginStateBlock();

		// ؽó Ҵ.		
		m_pd3dDevice->SetTexture(0, m_pTexture);
		
		if (D3DFONT_ZENABLE & m_dwFontFlags)
		// AND  true
			m_pd3dDevice->SetRenderState(D3DRS_ZENABLE, TRUE);
		// z ۸ ON
		else
			m_pd3dDevice->SetRenderState(D3DRS_ZENABLE, FALSE);
		
		//  ...
		m_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		m_pd3dDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pd3dDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
		m_pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
		m_pd3dDevice->SetRenderState(D3DRS_ALPHAREF, 0x08);
		m_pd3dDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
		m_pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
		m_pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
		m_pd3dDevice->SetRenderState(D3DRS_STENCILENABLE, FALSE);
		m_pd3dDevice->SetRenderState(D3DRS_CLIPPING, TRUE);
		m_pd3dDevice->SetRenderState(D3DRS_CLIPPLANEENABLE, FALSE);
		m_pd3dDevice->SetRenderState(D3DRS_VERTEXBLEND, D3DVBF_DISABLE);
		m_pd3dDevice->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE, FALSE);
		m_pd3dDevice->SetRenderState(D3DRS_FOGENABLE, FALSE);

		m_pd3dDevice->SetRenderState(D3DRS_COLORWRITEENABLE,
			D3DCOLORWRITEENABLE_RED | D3DCOLORWRITEENABLE_GREEN |
			D3DCOLORWRITEENABLE_BLUE | D3DCOLORWRITEENABLE_ALPHA);

		m_pd3dDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
		m_pd3dDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		m_pd3dDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
		m_pd3dDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
		m_pd3dDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
		m_pd3dDevice->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
		m_pd3dDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
		m_pd3dDevice->SetTextureStageState(0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE);
		m_pd3dDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
		m_pd3dDevice->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
		m_pd3dDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_POINT);
		m_pd3dDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_POINT);
		m_pd3dDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_NONE);

		// witch 0̸, ̽ ġ   ϰ, ڵ  .
		if (which == 0)
			m_pd3dDevice->EndStateBlock(&m_pStateBlockSaved);
		else
			m_pd3dDevice->EndStateBlock(&m_pStateBlockDrawText);

	}

	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: DrawText()
// Desc: Draws 2D text
//-----------------------------------------------------------------------------
HRESULT CD3DFont::DrawText(FLOAT sx, FLOAT sy, DWORD dwColor,
	const CHAR* strText, DWORD dwFlags)
{
	// ٹ̽ NULL̸ .
	if (m_pd3dDevice == NULL)
		return E_FAIL;

	// Setup renderstate
	// ġ   .
	m_pStateBlockSaved->Capture();

	//    ġ .
	m_pStateBlockDrawText->Apply();

	// FVF .
	m_pd3dDevice->SetFVF(D3DFVF_FONT2DVERTEX);

	//    ȼ ̴ NULL . 
	m_pd3dDevice->SetPixelShader(NULL);

	//  ۸ ġ  Ʈ ε.  m_pVB .
	m_pd3dDevice->SetStreamSource(0, m_pVB, 0, sizeof(FONT2DVERTEX));

	// Set filter states
	if (dwFlags & D3DFONT_FILTERED)
	{
		//  .
		m_pd3dDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
		m_pd3dDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	}

	FLOAT fStartX = sx;

	// Fill vertex buffer
	FONT2DVERTEX* pVertices = NULL;
	DWORD        dwNumTriangles = 0;
	m_pVB->Lock(0, 0, (void**)&pVertices, D3DLOCK_DISCARD);

	while (*strText)
	{
		// ޹ ڿ .
		TCHAR c = *strText++;

		if (c == _T('\n'))
		{
			// ٳѱ ,
			//  ٸŭ ؼ 缭 ϴ .
			sx = fStartX;
			sy += (m_fTexCoords[0][3] - m_fTexCoords[0][1])*m_dwTexHeight;
		}

		// (c-32) 0 ۰ų, 96 ũų  continue.
		if ((c - 32) < 0 || (c - 32) >= 128 - 32)
			continue;
		
		//  xy .
		FLOAT tx1 = m_fTexCoords[c - 32][0];
		FLOAT ty1 = m_fTexCoords[c - 32][1];
		FLOAT tx2 = m_fTexCoords[c - 32][2];
		FLOAT ty2 = m_fTexCoords[c - 32][3];

		// ũ   ؼ   .
		FLOAT w = (tx2 - tx1) *  m_dwTexWidth / m_fTextScale;
		FLOAT h = (ty2 - ty1) * m_dwTexHeight / m_fTextScale;

		if (c != _T(' '))
		{
			*pVertices++ = InitFont2DVertex(D3DXVECTOR4(sx + 0 - 0.5f, sy + h - 0.5f, 0.9f, 1.0f), dwColor, tx1, ty2);
			*pVertices++ = InitFont2DVertex(D3DXVECTOR4(sx + 0 - 0.5f, sy + 0 - 0.5f, 0.9f, 1.0f), dwColor, tx1, ty1);
			*pVertices++ = InitFont2DVertex(D3DXVECTOR4(sx + w - 0.5f, sy + h - 0.5f, 0.9f, 1.0f), dwColor, tx2, ty2);
			*pVertices++ = InitFont2DVertex(D3DXVECTOR4(sx + w - 0.5f, sy + 0 - 0.5f, 0.9f, 1.0f), dwColor, tx2, ty1);
			*pVertices++ = InitFont2DVertex(D3DXVECTOR4(sx + w - 0.5f, sy + h - 0.5f, 0.9f, 1.0f), dwColor, tx2, ty2);
			*pVertices++ = InitFont2DVertex(D3DXVECTOR4(sx + 0 - 0.5f, sy + 0 - 0.5f, 0.9f, 1.0f), dwColor, tx1, ty1);

			//   , +2 ﰢ 2(簢)
			dwNumTriangles += 2;

			if (dwNumTriangles * 3 > (MAX_NUM_VERTICES - 6))
			{
				// Unlock, render, and relock the vertex buffer
				m_pVB->Unlock();
				m_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, dwNumTriangles);
				pVertices = NULL;
				m_pVB->Lock(0, 0, (void**)&pVertices, D3DLOCK_DISCARD);
				dwNumTriangles = 0L;
			}
		}
		sx += w;
	}

	// Unlock and render the vertex buffer
	m_pVB->Unlock();

	if (dwNumTriangles > 0)
		m_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, dwNumTriangles);

	// Restore the modified renderstates
	//   ¸ .
	m_pStateBlockSaved->Apply();
	
	return S_OK;
}



